import json
import requests
import multistream.nginx as nginx
from multistream.logger import logger

class RtmpApplication:

    name : str = "invalid"
    service_type : str = "custom"
    _stream_key : str = ""
    _url : str = ""

    def __init__(self, name, url, key = "") -> None:
        self.name = name
        self._url = url
        self._stream_key = key
    
    @classmethod
    def new(cls, raw_data : dict):
        return RtmpApplication(raw_data["name"], raw_data["server"], raw_data["stream_key"])

    @property
    def stream_key(self) -> str:
        return self._stream_key

    def get_url(self) -> str:
        return self._url
    
    def get_nginx_app(self) -> dict:
        return nginx.generate_app(self.name, self.get_url(), self.stream_key)


class RtmpTwitch(RtmpApplication):
    name = "twitch"
    service_type = "twitch"
    
    def __init__(self, stream_key) -> None:
        self._stream_key = stream_key

    @classmethod
    def new(cls, raw_data : dict):
        new_app = RtmpTwitch(raw_data["stream_key"])
        if "name" in raw_data:
            new_app.name = raw_data["name"]
        return new_app

    def get_ingestion_server(self) -> str:
        r = requests.get("https://ingest.twitch.tv/ingests")
        if r.status_code == 200:
            data = r.json()
            if "ingests" in data:
                if len(data["ingests"]) > 0:
                    return data["ingests"][0]["url_template"]
        logger.warn("Failed to get ingestion server! Using default.")
        return "rtmp://ord02.contribute.live-video.net/app/{stream_key}"

    def get_url(self) -> str:
        return self.get_ingestion_server().replace("{stream_key}", "")

services = [RtmpApplication, RtmpTwitch]


from curses import raw
import imp
import json
from multiprocessing.dummy import Array
import apps
def load_config(config_string) -> Array:
    data = []

    json_data = json.loads(config_string)
    if "Apps" in json_data:
        for raw_app in json_data["Apps"]:
            if "platform" in raw_app:
                platform = raw_app["platform"]
                name = raw_app["id"] if "id" in raw_app else None
                url = raw_app["url"] if "url" in raw_app else None
                key = raw_app["key"] if "key" in raw_app else None
                if platform == "twitch":
                    data.append(apps.RtmpTwitch(key))
                elif platform == "custom":
                    if not url or not name:
                        continue
                    data.append(apps.RtmpApplication(name, url, key))

    return data


import crossplane
from multistream.logger import logger

def nginx(directive, args=[], block=None) -> dict:
    data = {
        "directive": directive,
        "args": [args] if isinstance(args, str) else args,   
    }
    if block != None:
        data["block"] = block
    
    return data

def generate_app(id, url, streamkey) -> dict:
    return nginx("application", f"stream_{id}",
    [
        nginx("live", "on"),
        nginx("record", "off"),
        nginx("allow", ["publish", "127.0.0.1"]),
        nginx("deny", ["publish", "all"]),
        nginx("push", f"{url}{streamkey}")
    ])

def build_config(apps = []) -> dict:

    push_to_apps = []
    nginx_apps = []
    for app in apps:
        logger.info(f"Generating config for '{app.name}'...")
        nginx_apps.append(app.get_nginx_app())
        push_to_apps.append(
            nginx("push", f"rtmp://localhost/stream_{app.name}")
        )

    nginx_config = [nginx("rtmp", [], [
        nginx("server", [],
        [
            nginx("listen", "1935"),
            nginx("chunk_size", "4096"),
            nginx("notify_method", "get"),
            nginx("application", "livestream",
            [
                nginx("live", "on"),
                nginx("record", "off"),
            ] + push_to_apps),
        ] + nginx_apps)
    ])]

    return crossplane.build(nginx_config)


import logging

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)


ch = logging.StreamHandler()
ch.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)

logger.addHandler(ch)

def enable_debug_logger() -> None:
    logger.setLevel(logging.DEBUG)
    ch.setLevel(logging.DEBUG)


import os, subprocess
import yaml
import multistream.apps as apps, multistream.nginx as nginx
from multistream.logger import logger

version = "0.0.1"

CONFIG_FILE : str = "config.yaml"
CONFIG_API_VERSION : str = "1.0"

def cli() -> None:
    main()

def main() -> None:
    rtmp_apps = []
    config_data : dict = {}

    if os.path.isfile(CONFIG_FILE):
        with open(CONFIG_FILE, "r") as f:
            config_data = yaml.load(f, Loader=yaml.SafeLoader)
    else:
        logger.error("Config file not found!")
        with open(CONFIG_FILE, "w") as f:
            config_data = {
                "apiVersion": CONFIG_API_VERSION,
                "kind": "MultistreamConfig",
                "apps": [
                    {
                        "service": "twitch",
                        "stream_key": "test"
                    }
                ]
            }
            yaml.dump(config_data, f)
            logger.info("Created default config file!")
        exit(1)

    if not "apiVersion" in config_data or config_data["apiVersion"] != CONFIG_API_VERSION:
        logger.error("Configuration file is out of date. Please update the configuration file.")
        exit(1)
    
    if not "kind" in config_data or config_data["kind"] != "MultistreamConfig":
        logger.error("Configuration file is not a 'MultistreamConfig'.")
        exit(1)

    if not "apps" in config_data:
        logger.error("Configuration file does not contain an 'apps' key.")
        exit(1)
    
    for app in config_data["apps"]:
        if not "service" in app:
            logger.warn("App does not contain a 'service' key.")
            continue
        if not app["service"] in (a.service_type for a in apps.services):
            logger.warn(f"Service '{app['service']}' is not a valid service.")
            continue

        service = next(a for a in apps.services if a.service_type == app["service"])

        try:
            rtmp_app = service.new(app)
        except KeyError as e:
            logger.error(f"{app['service'].capitalize()} app is missing a '{e.args[0]}' key.")
            continue

        if rtmp_app:
            rtmp_apps.append(rtmp_app)
        else:
            logger.warn(f"Failed to create app for service '{app['service']}'.")
            continue

    config_directory = os.environ.get("NGINX_CONFIG_DIRECTORY", ".")

    config = nginx.build_config(rtmp_apps)
    with open(os.path.join(config_directory, "test.conf"), "w") as f:
        f.write(config)

    logger.info("Configuration complete, starting NGINX!")
    
    try:
        subprocess.run(["nginx", "-g", "daemon off;"])
    except FileNotFoundError:
        logger.error("NGINX not found!")
        exit(1)
    except KeyboardInterrupt:
        pass

if __name__ == "__main__":
    main()



import re 
from os import path

from setuptools import setup
from codecs import open

app_name="multistream"
app_package="multistream"
app_requirements=["requests", "crossplane", "pyyaml"]

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

def read(*parts):
    return open(path.join(here, *parts), 'r').read()

def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^version = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")

setup(
    name=app_name,
    version=find_version(app_package,'__init__.py'),
    description='Python application',
    long_description=long_description,
    long_description_content_type="text/markdown",

    # The project's main homepage.
    url='https://github.com/meteoritesolutions/multistream',

    # Author details
    author='Meteorite Online Solutions, LLC',
    author_email='engineering@meteoritesolutions.com',

    # Choose your license
    license='Proprietary',

    # See https://pypi.org/classifiers/
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'Programming Language :: Python :: 3.9'
    ],
    keywords='Python',
    packages=[app_package],
    install_requires=app_requirements,
    package_data={},
    entry_points={
        'console_scripts' : [
            f"{app_name}={app_package}:cli"
        ]
    }
)